library tivio;
import 'package:meta/meta.dart';
import 'dart:async';
import 'bridge.dart';
import 'dto.dart';
import 'player/tivio_player_wrapper.dart';
import 'utils/enum.dart';
export 'mixins/tivio_widget.dart';
export 'mixins/tivio_channels.dart';
export 'mixins/tivio_sections.dart';
export 'mixins/tivio_videos.dart';
export 'dto.dart';
export 'player/tivio_player.dart';
export 'player/tivio_player_controller.dart';
export 'player/tivio_player_input_source.dart';
export 'player/tivio_player_wrapper.dart';

enum TivioLanguages {
  cs,
  en,
  sk,
  de,
  pl,
}

class Tivio {
  final String _appId;
  final TivioLanguages _language;
  final bool _development;
  final Bridge _bridge = Bridge();

  Tivio(this._appId, this._language, this._development);

  Future<Tivio> init() async {
    await _bridge.init(_appId, enumToString(_language), this._development);
    return this;
  }

  setLanguage(TivioLanguages language) => _bridge.setLanguage(enumToString(language));

  Future<TivioPlayerWrapper> getPlayer(String id) => _bridge.getPlayer(id);

  Future<TivioDataWidget> getWidgetById(String id) => _bridge.getWidgetById(id);

  Future<TivioDataChannel> getChannelById(String id) => _bridge.getChannelById(id);

  Future<TivioDataSection> getSectionById(String id) => throw new Exception('Not implemented yet');

  Future<TivioDataVideo> getVideoById(String id) => _bridge.getVideoById(id);

  subscribeToWidgets(String widgetId, Function(TivioDataWidget) callback) {
    return _bridge.subscribeToWidgets(widgetId, callback);
  }

  unsubscribeToWidgets() {
    _bridge.unsubscribeToWidgets();
  }

  /// API 1.0.0

  /// Widget

  Future<TivioDataStream<TivioDataWidget>> subscribeToWidget({@required String widgetId, int count = 10}) {
    return _bridge.subscribeToWidget(widgetId: widgetId, count: count);
  }

  /// Channel

  Future<TivioDataStream<TivioDataPagination<TivioDataChannel>>> subscribeToChannelsInWidget({@required String widgetId, int count = 10}) {
    return _bridge.subscribeToChannelsInWidget(widgetId: widgetId, count: count);
  }

  fetchNextChannelsInWidget({@required String widgetId, int count = 10}) {
    return _bridge.fetchNextChannelsInWidget();
  }

  Future<TivioDataStream<TivioDataChannel>> subscribeToChannel({@required String channelId}) {
    return _bridge.subscribeToChannel(channelId: channelId);
  }

  /// Section

  Future<TivioDataStream<TivioDataPagination<TivioDataSection>>> subscribeToSectionsInChannel({@required String channelId, int count = 10}) {
    return _bridge.subscribeToSectionsInChannel(channelId: channelId, count: count);
  }

  fetchNextSectionsInChannel({@required String channelId, int count = 10}) {
    return _bridge.fetchNextSectionsInChannel();
  }

  Future<TivioDataStream<TivioDataSection>> subscribeToSection({@required String sectionId}) {
    return _bridge.subscribeToSection(sectionId: sectionId);
  }

  /// Video

  Future<TivioDataStream<TivioDataPagination<TivioDataVideo>>> subscribeToVideosInSection ({@required String sectionId, int count = 10}) {
    return _bridge.subscribeToVideosInSection(sectionId: sectionId, count: count);
  }

  fetchNextVideosInSection({@required String sectionId, int count = 10}) {
    return _bridge.fetchNextVideosInSection();
  }

  Future<TivioDataStream<TivioDataVideo>> subscribeToVideo({@required String videoId}) {
    return _bridge.subscribeToVideo(videoId: videoId);
  }

  /// Recent

  Future<TivioDataPagination<TivioDataChannel>> searchChannels({@required String query, int limit = 10}) {
    return _bridge.searchChannels(query, limit);
  }

  Future<TivioDataPagination<TivioDataVideo>> searchVideos({@required String query, int limit = 10}) {
    return _bridge.searchVideos(query, limit);
  }
}

Future<Tivio> initTivio({String secret, TivioLanguages language, bool development = false}) {
  return Tivio(secret, language, development).init();
}
